# О программе #

Программа для преобразования чеков от ФНС в формат QIF (Quicken Interchange
Format), который подходит для последующего импорта в
[GnuCash](http://gnucash.org) или другую программу учёта финансов.
Умеет классифицировать товары по категориям, в том числе автоматически по
заранее подготовленным шаблонам.

Чеки можно получить в формате JSON через программу
[Проверка касcового чека](https://play.google.com/store/apps/details?id=ru.fns.billchecker).
Экспортировать можно как отдельные чеки (через пункт «Поделиться»), так и все
чеки за указанный период времени (через пункт меню «Получение выписки»).

Автор: А.Галанин <al@galanin.nnov.ru>

# Использование #

Программа поддерживает два режима работы:

* **Интерактивный** (по умолчанию) - открывается окно, в котором можно
  классифицировать отдельные товары каждого чека по категориям.
* **Автоматизированный** - товары из чеков классифицируются по заранее
  установленным шаблонам. Если для какого-то товара подходящий шаблон не найден,
  то программа завершится с ошибкой.

Справку можно получить по параметру командной строки _-help_:

    bill2qif [настройки] [входной файл] [выходной файл]
    настройки:
     -interactive         Интерактивный режим (по умолчанию)
     -automated           Автоматизированный режим, выход при неизвестном названии товара
     -config value        Имя файла настроек <$HOME/.config/bill2qif.conf>
     -version             Напечатать версию и выйти
     --                   Прекратить обработку настроек
     -help                Напечатать справку
     -?                   Напечатать справку

Если входной/выходной файлы не заданы, то в автоматизированном режиме будут
использоваться стандартный вход и выход соответственно. В интерактивном режиме
будет выдан запрос на выбор файлов.

# Классификация товаров в интерактивном режиме #

Для классификации товара надо выбрать подходящую категорию в одноимённом
столбце. Если её нет - ввести вручную.

Можно задать шаблон классификации для любого товара в столбце «Шаблон
классификации». Шаблон может содержать символы \* и ?, обозначающие произвольное
количество любых символов или произвольный одиночный символ.

*Пример*: под шаблон «\*масло\*» подойдёт товар «Сливочное масло 100г».

Заданный шаблон сохраняется, и впоследствии будет использован для классификации
товаров в новых чеках. Отредактировать список шаблонов можно в конфигурационном
файле.

Некоторые магазины не отправляют в ФНС своё название, либо их появляется желание
обозначить магазин более конкретно (например, «Булочная у дома»). Задать
предпочитаемое название можно через поле «Магазин».

# Горячие клавиши в интерактивном режиме #

Клавиша         | Действие
----------------|-----------------------------------------------------
Enter           | Перейти в режим редактирования категории или шаблона
Delete          | Очистить заданное значение категории или шаблона
Ctrl-Tab        | Перейти к следующему чеку
Ctrl-Shift-Tab  | Перейти к предыдущему чеку
Ctrl-N          | Перейти к следующему неклассифицированному товару
Ctrl-L          | Редактировать название магазина
Alt-F4          | Завершить работу

# Конфигурационный файл #

Файл конфигурации называется _.bill2qif.conf_ и хранится к домашнем каталоге
пользователя (на который указывает переменная окружения HOME).

В секции _config_ хранится информация о версии формата файла. В данный момент
1.0.

    [config]
    format_version=1.0

В секции _patterns_ перечислены все установленные соответствия между шаблоном
названия товара и категорией в формате _шаблон=категория_:

    [patterns]
    *БАНАНЫ*=Продукты
    *груши*=Продукты
    *ГУБКИ ДЛЯ ПОСУДЫ*=Хозтовары
    *НОСКИ*=Одежда

В секции _shops_ перечислены соответствия между ИНН юридического лица и
настроенным именем магазина:

    [shops]
    1234567890=Булочная у дома

# Сборка и установка #

## Linux ##

Вам понадобится Tcl (не ниже версии 8.6) и следующие библиотеки: Tk (не ниже 8.6),
tcllib (не ниже 1.18; используются библиотеки msgcat, cmdline, control, json, struct,
inifile) и tklib (не ниже 0.6; используются библиотеки autoscroll и tablelist).
Поставьте их из репозитория своего дистрибутива.

Можно запускать файл _bill2qif.tcl_ непосредственно из дерева исходных кодов. Или
же позвать команду «make install» для установки. Параметры для make
соответствуют [соглашениям проекта GNU](https://www.gnu.org/prep/standards/html_node/Makefile-Conventions.html).

## Windows ##

Возьмите исполняемый exe-файл из списка загрузок.

## Сборка starpack для Windows ##

Это нужно только в том случае, если собранный мной exe-шник по какой-то причине
не понравился и захотелось сделать всё самостоятельно. Делается только из-под Linux.

1. Установите библиотеки, перечисленные в разделе «Linux». 
1. Скачайте необходимые файлы (пути указаны примерно, обновите под ваши
   требования):

       * tclkit (cli) под свою платформу - https://storage.googleapis.com/google-code-archive-downloads/v2/code.google.com/tclkit/tclkitsh-8.5.9-linux-ix86.gz
       * sdx - https://storage.googleapis.com/google-code-archive-downloads/v2/code.google.com/tclkit/sdx-20110317.kit
       * tclkit (gui) под Windows - https://datapacket.dl.sourceforge.net/project/twapi/Tcl%20binaries/Tclkits/tclkit-gui-8_6_8-x86.exe

1. Передайте названия файлов и пути к библиотекам в make (см. внутри Makefile).
1. make starpack
