package require Tcl 8.6
package require msgcat 1.5

package provide converter 1.0

namespace eval converter {

    proc convertSplit {data} {
        set res {}

        set quantity [dict get $data quantity]
        set note [dict get $data text]
        if {$quantity != 1} {
            set note [msgcat::mc "%s (%s units)" $note $quantity]
        }

        lappend res "S[dict get $data category]"
        lappend res "E$note"
        lappend res [format {$%.2f} [expr {-[dict get $data sum]}]]

        join $res \n
    }

    proc convertBill {bill} {
        set res {}

        # bill header
        lappend res "!Type:CCard"
        lappend res "D[clock format [dict get $bill dateTime] -format {%d.%m.%y}]"
        lappend res [format {T%.2f} [expr {-[dict get $bill sum]}]]
        lappend res "P[dict get $bill shopName]"
        # splits
        foreach split [lreverse [dict get $bill splits]] {
            lappend res [convertSplit $split]
        }
        lappend res "^"

        join $res \n
    }

    proc convertBillList {bills} {
        set res {}

        # file header
        lappend res "!Account"
        lappend res "NCredit card"
        lappend res "TCCard"
        lappend res "^"
        # bills
        foreach bill $bills {
            lappend res [convertBill $bill]
        }

        join $res \n
    }

    namespace export *

}

# vim: set ft=tcl:

